/*
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *  See the NOTICE file distributed with this work for additional
 *  information regarding copyright ownership.
 */
package org.topbraid.shacl.util;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.topbraid.shacl.vocabulary.SH;

/**
 * A singleton to record (and possibly print) failures thrown by the validation engine.
 * Can be overloaded to install different failure handling.
 * 
 * @author Holger Knublauch
 */
public class FailureLog {
	
	private Logger logger = LoggerFactory.getLogger(FailureLog.class);

	private static FailureLog singleton = new FailureLog();
	
	public static FailureLog get() {
		return singleton;
	}
	
	public static void set(FailureLog value) {
		FailureLog.singleton = value;
	}
	
	
	public void logFailure(String message) {
		logger.error(SH.NAME + " Failure: " + message);
	}
	
	
	public void logWarning(String message) {
		logger.warn(SH.NAME + " Warning: " + message);
	}
}
